package im;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;
import java.util.Map;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;

public class Clients
{
	// Protect the constructor, so no other class can call it
	private Clients()
	{
		JSONArray json = getUsersFromIdM("424dc0b9cac7392e5fee91ba4f6dc32b");		
		
		if (json != null)
		{
			//add all users from IdM to clientList
			for (int i = 0; i < json.size(); i++)
			{
				JSONObject tmp = (JSONObject) json.get(i);
				clients.put(tmp.get("username").toString(), "none");
			}
		}
		else
		{
			System.out.println("JSONArray from IdM is NULL.");
		}
	}

	// Create the only instance, save it to a private static variable
	private static Clients instance = new Clients();

	// Make the static instance publicly available
	public static Clients getInstance() { return instance; }

	private HashMap<String, String> clients = new HashMap<String, String>();
	
	public int signInClient(String uid, String ip)
	{
		if (clients.containsKey(uid))
		{
			if (!clients.get(uid).equals(ip))
			{
				clients.remove(uid);
				clients.put(uid, ip);
				
				//signed in correctly
				return 0;
			}
			else
			{
				//user is signed in
				return 1;
			}
		}
		else
		{
			//cant sign in user - user not in IdM list
			return -1;
		}
	}
	
	public String getUsername(String ip)
	{
		for (Map.Entry<String, String> entry : this.clients.entrySet())
		{
			if(entry.getValue().equals(ip))
			{
				return entry.getKey();
			}
		}

		return "";
	}
	
	public int signOutClient(String uid)
	{
		if (clients.containsKey(uid))
		{
			if (!clients.get(uid).equals("none"))
			{
				clients.remove(uid);
				clients.put(uid, "none");
				
				//user successfully signed out
				return 0;
			}
			else
			{
				//user is not signed in
				return 1;
			}
			
		}
		else
		{
			//user not existing in context of IdM
			return -1;
		}
	}

	public HashMap<String, String> getClients()
	{
		return clients;
	}
	
	public Boolean hasClients()
	{
		if (clients.isEmpty())
		{
			return false;
		}
		else
		{
			return true;
		}
	}
	
	public JSONArray clientsToJSON()
	{
		JSONArray cl = new JSONArray();
		
		for (String key : this.clients.keySet())
		{
			JSONObject tmp = new JSONObject();
			tmp.put("username", key);
			tmp.put("ip", this.clients.get(key));
			
			cl.add(tmp);
		}

		return cl;
	}
	
	private JSONArray getUsersFromIdM(String token)
	{
		try
		{	
			URL idmUrl = new URL("http://hbbnext.ngidm.org/api/v2/users?access_token=" + token);
			URLConnection connection = idmUrl.openConnection();
			
			BufferedReader br = new BufferedReader(
                    new InputStreamReader(connection.getInputStream()));
			
			StringBuilder sb = new StringBuilder();
            String line;
            
            while ((line = br.readLine()) != null) {
                sb.append(line+"\n");
            }
            
            /*
             * [{"id":2,"alias":"father","username":"father","web_url":"http://hbbnext.ngidm.org/users/2","href":"http://hbbnext.ngidm.org/api/v2/users/2"},{"id":3,"alias":"mother","username":"mother","web_url":"http://hbbnext.ngidm.org/users/3","href":"http://hbbnext.ngidm.org/api/v2/users/3"},{"id":4,"alias":"son","username":"son","web_url":"http://hbbnext.ngidm.org/users/4","href":"http://hbbnext.ngidm.org/api/v2/users/4"},{"id":5,"alias":"friend1","username":"friend1","web_url":"http://hbbnext.ngidm.org/users/5","href":"http://hbbnext.ngidm.org/api/v2/users/5"},{"id":6,"alias":"friend2","username":"friend2","web_url":"http://hbbnext.ngidm.org/users/6","href":"http://hbbnext.ngidm.org/api/v2/users/6"},{"id":8,"alias":"oskar","username":"Oskar","web_url":"http://hbbnext.ngidm.org/users/8","href":"http://hbbnext.ngidm.org/api/v2/users/8"},{"id":9,"alias":"maria","username":"Maria","web_url":"http://hbbnext.ngidm.org/users/9","href":"http://hbbnext.ngidm.org/api/v2/users/9"}]
             */
            JSONArray json = (JSONArray)new JSONParser().parse(sb.toString());
            
            br.close();
            
            return json;
		}
		catch (Exception e)
		{
			return null;
		}		
	}
}